@@ -42,6 +42,8 @@
 
 -define(SERVER, ?MODULE).
 -define(ENV_APP_NAME, osmo_s1gw).
+-define(ENV_DEFAULT_SCTP_BUFSZ, 65536).
+-define(ENV_DEFAULT_SCTP_NODELAY, true).
 -define(ENV_DEFAULT_S1GW_BIND_ADDR, "127.0.1.1").
 -define(ENV_DEFAULT_S1GW_BIND_PORT, ?S1AP_PORT).
 -define(ENV_DEFAULT_MME_LOC_ADDR, "127.0.2.1").
@@ -59,17 +61,10 @@ start_link() ->
 
 
 init([]) ->
-    S1GWBindAddr = get_env(s1gw_bind_addr, ?ENV_DEFAULT_S1GW_BIND_ADDR),
-    S1GWBindPort = get_env(s1gw_bind_port, ?ENV_DEFAULT_S1GW_BIND_PORT),
-    MmeLocAddr = get_env(mme_loc_addr, ?ENV_DEFAULT_MME_LOC_ADDR),
-    MmeRemAddr = get_env(mme_rem_addr, ?ENV_DEFAULT_MME_REM_ADDR),
-    MmeRemPort = get_env(mme_rem_port, ?ENV_DEFAULT_MME_REM_PORT),
     PfcpLocAddr = get_env(pfcp_loc_addr, ?ENV_DEFAULT_PFCP_LOC_ADDR),
     PfcpRemAddr = get_env(pfcp_rem_addr, ?ENV_DEFAULT_PFCP_REM_ADDR),
 
-    SctpServer = {sctp_server, {sctp_server, start_link,
-                                [S1GWBindAddr, S1GWBindPort,
-                                 {{MmeLocAddr, MmeRemAddr}, MmeRemPort}]},
+    SctpServer = {sctp_server, {sctp_server, start_link, [server_cfg()]},
                   permanent,
                   5000,
                   worker,
@@ -92,4 +87,27 @@ init([]) ->
 get_env(Param, Default) ->
     application:get_env(?ENV_APP_NAME, Param, Default).
 
+
+-spec client_cfg() -> sctp_client:cfg().
+client_cfg() ->
+    SockOpts = #{sctp_nodelay => get_env(mme_nodelay, ?ENV_DEFAULT_SCTP_NODELAY),
+                 recbuf => get_env(mme_recbuf, ?ENV_DEFAULT_SCTP_BUFSZ),
+                 sndbuf => get_env(mme_sndbuf, ?ENV_DEFAULT_SCTP_BUFSZ)},
+    #{laddr => get_env(mme_loc_addr, ?ENV_DEFAULT_MME_LOC_ADDR),
+      raddr => get_env(mme_rem_addr, ?ENV_DEFAULT_MME_REM_ADDR),
+      rport => get_env(mme_rem_port, ?ENV_DEFAULT_MME_REM_PORT),
+      sockopts => maps:to_list(SockOpts)}.
+
+
+-spec server_cfg() -> sctp_server:cfg().
+server_cfg() ->
+    SockOpts = #{sctp_nodelay => get_env(s1gw_nodelay, ?ENV_DEFAULT_SCTP_NODELAY),
+                 recbuf => get_env(s1gw_recbuf, ?ENV_DEFAULT_SCTP_BUFSZ),
+                 sndbuf => get_env(s1gw_sndbuf, ?ENV_DEFAULT_SCTP_BUFSZ)},
+    #{laddr => get_env(s1gw_bind_addr, ?ENV_DEFAULT_S1GW_BIND_ADDR),
+      lport => get_env(s1gw_bind_port, ?ENV_DEFAULT_S1GW_BIND_PORT),
+      sockopts => maps:to_list(SockOpts),
+      handler => sctp_proxy,
+      priv => client_cfg()}.
+
 %% vim:set ts=4 sw=4 et:
