@@ -69,12 +69,18 @@
     ?S1GW_CTR_S1AP_PROXY_IN_PKT_PROC_ERROR,                 %% failed to process
     ?S1GW_CTR_S1AP_PROXY_IN_PKT_ERAB_SETUP_REQ,             %% E-RAB SETUP.req PDUs
     ?S1GW_CTR_S1AP_PROXY_IN_PKT_ERAB_SETUP_RSP,             %% E-RAB SETUP.rsp PDUs
+    ?S1GW_CTR_S1AP_PROXY_IN_PKT_ERAB_MODIFY_REQ,            %% E-RAB MODIFY.req PDUs
+    ?S1GW_CTR_S1AP_PROXY_IN_PKT_ERAB_MODIFY_RSP,            %% E-RAB MODIFY.rsp PDUs
     ?S1GW_CTR_S1AP_PROXY_IN_PKT_ERAB_RELEASE_CMD,           %% E-RAB RELEASE.cmd PDUs
     ?S1GW_CTR_S1AP_PROXY_IN_PKT_ERAB_RELEASE_RSP,           %% E-RAB RELEASE.rsp PDUs
     ?S1GW_CTR_S1AP_PROXY_IN_PKT_ERAB_RELEASE_IND,           %% E-RAB RELEASE.ind PDUs
     ?S1GW_CTR_S1AP_PROXY_IN_PKT_ERAB_MOD_IND,               %% E-RAB MODIFY.ind PDUs
+    ?S1GW_CTR_S1AP_PROXY_IN_PKT_ERAB_MOD_CNF,               %% E-RAB MODIFY.cnf PDUs
     ?S1GW_CTR_S1AP_PROXY_IN_PKT_INIT_CTX_REQ,               %% INITIAL CONTEXT SETUP.req PDUs
     ?S1GW_CTR_S1AP_PROXY_IN_PKT_INIT_CTX_RSP,               %% INITIAL CONTEXT SETUP.rsp PDUs
+    ?S1GW_CTR_S1AP_PROXY_IN_PKT_RELEASE_CTX_REQ,            %% UE CONTEXT RELEASE.req PDUs
+    ?S1GW_CTR_S1AP_PROXY_IN_PKT_RELEASE_CTX_CMD,            %% UE CONTEXT RELEASE.cmd PDUs
+    ?S1GW_CTR_S1AP_PROXY_IN_PKT_RELEASE_CTX_COMPL,          %% UE CONTEXT RELEASE.compl PDUs
     %% s1ap_proxy: OUTgoing PDU counters
     ?S1GW_CTR_S1AP_PROXY_OUT_PKT_FWD_ALL,                   %% forwarded: total
     ?S1GW_CTR_S1AP_PROXY_OUT_PKT_FWD_PROC,                  %% forwarded: processed
@@ -92,29 +98,14 @@
 -type counter() :: [ctr | _].
 -type gauge() :: [gauge | _].
 
--spec new_ctr(counter()) -> ok.
-new_ctr(Name) ->
-    %%?LOG_INFO("New counter ~p", [Name]),
-    ok = exometer:new(Name, counter).
 
--spec new_ctrs(list(counter())) -> ok.
-new_ctrs([]) ->
-    ok;
-new_ctrs([Name | MoreNames]) ->
-    new_ctr(Name),
-    new_ctrs(MoreNames).
+-spec register_all(Type, List) -> Result
+    when Type :: exometer:type(),
+         List :: list(exometer:name()),
+         Result :: list(exometer:name()).
+register_all(Type, List) ->
+    lists:filter(fun(Name) -> exometer:new(Name, Type) =/= ok end, List).
 
--spec new_gauge(gauge()) -> ok.
-new_gauge(Name) ->
-    %%?LOG_INFO("New gauge ~p", [Name]),
-    ok = exometer:new(Name, gauge).
-
--spec new_gauges(list(gauge())) -> ok.
-new_gauges([]) ->
-    ok;
-new_gauges([Name | MoreNames]) ->
-    new_gauge(Name),
-    new_gauges(MoreNames).
 
 -spec get_current_value(counter() | gauge()) -> integer().
 get_current_value(Name) ->
@@ -122,14 +113,16 @@ get_current_value(Name) ->
     {ok, [{value, PrevVal}]} = Result,
     PrevVal.
 
+
 %% ------------------------------------------------------------------
 %% public API
 %% ------------------------------------------------------------------
 
 init() ->
     ?LOG_INFO("Initiating metrics"),
-    new_ctrs(?S1GW_COUNTERS),
-    new_gauges(?S1GW_GAUGES).
+    logger:set_module_level(exometer_report_statsd, info),
+    [] = register_all(counter, ?S1GW_COUNTERS),
+    [] = register_all(gauge, ?S1GW_GAUGES).
 
 %%%%%%%%%%%%%
 %% CTR APIs
