@@ -34,6 +34,7 @@
 
 -module(sctp_proxy).
 -behaviour(gen_statem).
+-behaviour(sctp_client).
 
 -export([init/1,
          callback_mode/0,
@@ -41,8 +42,9 @@
          connected/3,
          code_change/4,
          terminate/3]).
--export([start_link/3,
+-export([start_link/2,
          send_data/2,
+         fetch_info/1,
          shutdown/1]).
 
 -include_lib("kernel/include/logger.hrl").
@@ -55,11 +57,12 @@
 %% public API
 %% ------------------------------------------------------------------
 
--spec start_link(gen_sctp:assoc_id(),
-                 sctp_client:loc_rem_addr(),
-                 inet:port_number()) -> gen_statem:start_ret().
-start_link(Aid, MmeAddr, MmePort) ->
-    gen_statem:start_link(?MODULE, [Aid, MmeAddr, MmePort], []).
+-spec start_link(Aid, ConnCfg) -> Result
+    when Aid :: gen_sctp:assoc_id(),
+         ConnCfg :: sctp_client:cfg(),
+         Result :: gen_statem:start_ret().
+start_link(Aid, ConnCfg) ->
+    gen_statem:start_link(?MODULE, [Aid, ConnCfg], []).
 
 
 -spec send_data(pid(), binary()) -> ok.
@@ -67,6 +70,11 @@ send_data(Pid, Data) ->
     gen_statem:cast(Pid, {send_data, Data}).
 
 
+-spec fetch_info(pid()) -> proplists:proplist().
+fetch_info(Pid) ->
+    gen_statem:call(Pid, ?FUNCTION_NAME).
+
+
 -spec shutdown(pid()) -> ok.
 shutdown(Pid) ->
     gen_statem:stop(Pid).
@@ -76,14 +84,13 @@ shutdown(Pid) ->
 %% gen_statem API
 %% ------------------------------------------------------------------
 
-init([Aid, MmeAddr, MmePort]) ->
-    process_flag(trap_exit, true),
+init([Aid, ConnCfg]) ->
+    {ok, Pid} = s1ap_proxy:start_link(),
     {ok, connecting,
      #{enb_aid => Aid,
-       mme_addr => MmeAddr,
-       mme_port => MmePort,
+       conn_cfg => ConnCfg,
        tx_queue => [],
-       priv => s1ap_proxy:init()}}.
+       handler => Pid}}.
 
 
 callback_mode() ->
@@ -92,10 +99,10 @@ callback_mode() ->
 
 %% CONNECTING state
 connecting(enter, OldState,
-           #{mme_addr := MmeAddr, mme_port := MmePort} = S) ->
+           #{conn_cfg := ConnCfg} = S) ->
     ?LOG_INFO("State change: ~p -> ~p", [OldState, ?FUNCTION_NAME]),
     %% Initiate connection establishment with the MME
-    {ok, Sock} = sctp_client:connect(MmeAddr, MmePort),
+    {ok, Sock} = sctp_client:connect(ConnCfg),
     {next_state, connecting, S#{sock => Sock},
      [{state_timeout, 2_000, conn_est_timeout}]};
 
@@ -132,8 +139,7 @@ connecting(info, {sctp, _Socket, MmeAddr, MmePort,
     {keep_state, S};
 
 connecting(Event, EventData, S) ->
-    ?LOG_ERROR("Unexpected event ~p: ~p", [Event, EventData]),
-    {keep_state, S}.
+    handle_event(?FUNCTION_NAME, Event, EventData, S).
 
 
 %% CONNECTED state
@@ -145,8 +151,8 @@ connected(enter, OldState, S0) ->
 
 %% Handle an eNB -> MME data forwarding request (forward)
 connected(cast, {send_data, Data}, S0) ->
-    S1 = sctp_send(Data, S0),
-    {keep_state, S1};
+    sctp_send(Data, S0),
+    {keep_state, S0};
 
 %% Handle an #sctp_assoc_change event (MME connection state)
 connected(info, {sctp, _Socket, MmeAddr, MmePort,
@@ -164,28 +170,25 @@ connected(info, {sctp, _Socket, MmeAddr, MmePort,
 
 %% Handle an #sctp_sndrcvinfo event (MME -> eNB data)
 connected(info, {sctp, _Socket, MmeAddr, MmePort,
-                 {[#sctp_sndrcvinfo{assoc_id = Aid}], Data}},
+                 {[#sctp_sndrcvinfo{assoc_id = Aid,
+                                    stream = SID,
+                                    ssn = SSN,
+                                    tsn = TSN}], Data}},
           #{sock := Sock,
             enb_aid := EnbAid,
             mme_aid := Aid,
-            priv := Priv} = S) ->
+            handler := Pid} = S) ->
     ?LOG_DEBUG("MME connection (id=~p, ~p:~p) -> eNB: ~p",
-               [Aid, MmeAddr, MmePort, Data]),
-    {Action, NewPriv} = s1ap_proxy:process_pdu_safe(Data, Priv),
-    case Action of
+               [Aid, MmeAddr, MmePort,
+                #{tsn => TSN, sid => SID, ssn => SSN,
+                  len => byte_size(Data), data => Data}]),
+    case s1ap_proxy:process_pdu(Pid, Data) of
         {forward, FwdData} ->
             sctp_server:send_data(EnbAid, FwdData);
         {reply, ReData} ->
-            ok = sctp_client:send_data({Sock, Aid}, ReData)
+            ok = sctp_common:send_data({Sock, Aid}, ReData)
     end,
-    {keep_state, S#{priv := NewPriv}};
-
-%% Handle termination events of the child processes
-connected(info, {'EXIT', Pid, Reason},
-          #{priv := Priv} = S) ->
-    ?LOG_DEBUG("Child process ~p terminated with reason ~p", [Pid, Reason]),
-    NewPriv = s1ap_proxy:handle_exit(Pid, Priv),
-    {keep_state, S#{priv := NewPriv}};
+    {keep_state, S};
 
 %% Catch-all for other kinds of SCTP events
 connected(info, {sctp, _Socket, MmeAddr, MmePort,
@@ -194,25 +197,39 @@ connected(info, {sctp, _Socket, MmeAddr, MmePort,
                [MmeAddr, MmePort, AncData, Data]),
     {keep_state, S};
 
-%% Catch-all handler for this state
 connected(Event, EventData, S) ->
-    ?LOG_ERROR("Unexpected event ~p: ~p", [Event, EventData]),
-    {keep_state, S}.
+    handle_event(?FUNCTION_NAME, Event, EventData, S).
 
 
+%% Event handler for all states
+handle_event(State, {call, From}, fetch_info,
+             #{handler := Pid} = S0) ->
+    ERABs = s1ap_proxy:fetch_erab_list(Pid),
+    S1 = maps:with([enb_aid, mme_aid, mme_addr, mme_port], S0),
+    S2 = S1#{state => State, erab_list => ERABs},
+    Reply = proplists:from_map(S2),
+    {keep_state_and_data, {reply, From, Reply}};
+
+handle_event(State, {call, From}, EventData, _S) ->
+    ?LOG_ERROR("Unexpected call in state ~p: ~p", [State, EventData]),
+    {keep_state_and_data, {reply, From, {error, {unexpected_call, State}}}};
+
+handle_event(State, Event, EventData, _S) ->
+    ?LOG_ERROR("Unexpected event ~p in state ~p: ~p", [Event, State, EventData]),
+    keep_state_and_data.
+
 
 code_change(_Vsn, State, S, _Extra) ->
     {ok, State, S}.
 
 
-terminate(Reason, State, S) ->
+terminate(Reason, State, #{handler := Pid} = S) ->
     ?LOG_NOTICE("Terminating in state ~p, reason ~p", [State, Reason]),
+    s1ap_proxy:shutdown(Pid),
     case S of
         #{sock := Sock,
-          mme_aid := Aid,
-          priv := Priv} ->
-            s1ap_proxy:deinit(Priv),
-            sctp_client:disconnect({Sock, Aid}),
+          mme_aid := Aid} ->
+            sctp_common:shutdown({Sock, Aid}),
             gen_sctp:close(Sock);
         #{sock := Sock} ->
             gen_sctp:close(Sock);
@@ -229,25 +246,23 @@ sctp_send(Data,
           #{sock := Sock,
             enb_aid := EnbAid,
             mme_aid := Aid,
-            priv := Priv} = S) ->
-    {Action, NewPriv} = s1ap_proxy:process_pdu_safe(Data, Priv),
-    case Action of
+            handler := Pid}) ->
+    case s1ap_proxy:process_pdu(Pid, Data) of
         {forward, FwdData} ->
-            ok = sctp_client:send_data({Sock, Aid}, FwdData);
+            ok = sctp_common:send_data({Sock, Aid}, FwdData);
         {reply, ReData} ->
             sctp_server:send_data(EnbAid, ReData)
-    end,
-    S#{priv := NewPriv}.
+    end.
 
 
 %% Send pending messages to the MME
 sctp_send_pending(#{tx_queue := Pending} = S) ->
     sctp_send_pending(lists:reverse(Pending), S).
 
-sctp_send_pending([Data | Pending], S0) ->
-    S1 = sctp_send(Data, S0),
+sctp_send_pending([Data | Pending], S) ->
+    sctp_send(Data, S),
     s1gw_metrics:gauge_dec(?S1GW_GAUGE_S1AP_PROXY_UPLINK_PACKETS_QUEUED),
-    sctp_send_pending(Pending, S1);
+    sctp_send_pending(Pending, S);
 
 sctp_send_pending([], S) ->
     S#{tx_queue := []}.
